/*-----------------------------------------------------------------*/
/*! 
  \file omlfileinputstream.h
  \brief Open Math Input Stream with file descriptor (int) definition.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __OMLFILEINPUTSTREAM_H__
#define __OMLFILEINPUTSTREAM_H__

#include "omlinputstream.h"
#include "omlfileio.h"

/*-----------------------------------------------------------------*/
/*!
    @class OML_Fileinputstream
    \brief  classe de base pour gerer un flux entrant vers un 
    descripteur de fichier de type int en  Open Math
*/
/*-----------------------------------------------------------------*/
class OML_Fileinputstream : public OML_inputstream
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   // ! use the file descriptor fd (return by open for example)
   OML_Fileinputstream() : OML_inputstream(), m_fd() {};
   OML_Fileinputstream(int fd, bool bsocket) : OML_inputstream(), m_fd(fd, bsocket) {};
   virtual ~OML_Fileinputstream() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! attach the stream to an opened file descriptor
   void SetFile(int fd) { m_fd.SetFile(fd, false); };
   void SetSocket(int fd) { m_fd.SetFile(fd, true); };
   int GetFile() const { return m_fd.GetFile(); };
   OML_Fileio& GetFileio() { return m_fd; };
   
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////
   //! read the xml document
   virtual bool ReadXmlDoc();  
   //! read a line
   virtual bool ReadLine(SCSCP_status* status);

private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   OML_Fileio m_fd; //!< file descriptor
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   //! set flag of errors to true if value = -1
   template<typename T> void CheckErrors(T value)
   {
    if (value==-1) SetError();
   }
};


#endif /*__OMLFILEINPUTSTREAM_H__*/
