/*-----------------------------------------------------------------*/
/*! 
  \file omlfileoutputstream.cxx
  \brief Open Math Output Stream with file descriptor (int) implemntation.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#if HAVE_CONFIG_H
#include "scscpconfig.h"
#endif /*HAVE_CONFIG_H*/
#include <stdio.h>
#if HAVE_STRING_H
#include <string.h>
#endif
#include "scscptypes.h"
#include "scscpinternal.h"
#include "omlfileoutputstream.h"



/*-----------------------------------------------------------------*/
/*! write characters to stream
 @param data  (in) value to write
*/
/*-----------------------------------------------------------------*/
bool OML_Fileoutputstream::WriteChars(const_OMLchar data)
{
 CheckErrors(m_fd.write(data, strlen(data))); 
 return hasnoerror();
}

/*-----------------------------------------------------------------*/
/*! write characters to stream
 @param data  (in) value to write
*/
/*-----------------------------------------------------------------*/
OML_outputstream& OML_Fileoutputstream::operator<<(const_OMLchar data)
{
 CheckErrors(m_fd.write(data, strlen(data))); 
 return *this;
}

/*-----------------------------------------------------------------*/
/*! write characters to stream
 @param data  (in) value to write
*/
/*-----------------------------------------------------------------*/
OML_outputstream& OML_Fileoutputstream::operator<<(size_t data)
{
 char sdata[256];
 sprintf(sdata, "%llu", (unsigned long long) data);
 CheckErrors(m_fd.write(sdata, strlen(sdata))); 
 return *this;
}

/*-----------------------------------------------------------------*/
/*! write characters to stream
 @param data  (in) value to write
*/
/*-----------------------------------------------------------------*/
OML_outputstream& OML_Fileoutputstream::operator<<(long long data)
{
 char sdata[256];
 sprintf(sdata, "%lld", data);
 CheckErrors(m_fd.write(sdata, strlen(sdata))); 
 return *this;
}

/*-----------------------------------------------------------------*/
/*! write characters to stream
 @param data  (in) value to write
*/
/*-----------------------------------------------------------------*/
OML_outputstream& OML_Fileoutputstream::operator<<(long data)
{
 char sdata[256];
 sprintf(sdata, "%ld", data);
 CheckErrors(m_fd.write(sdata, strlen(sdata))); 
 return *this;
}

/*-----------------------------------------------------------------*/
/*! write characters to stream
 @param data  (in) value to write
*/
/*-----------------------------------------------------------------*/
OML_outputstream& OML_Fileoutputstream::operator<<(int data)
{
 char sdata[256];
 sprintf(sdata, "%lld", (signed long long) data);
 CheckErrors(m_fd.write(sdata, strlen(sdata))); 
 return *this;
}

