/*-----------------------------------------------------------------*/
/*! 
  \file omlmpiinputstream.h
  \brief Open Math Input Stream with mpi stream.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/

#ifndef __OMLMPIINPUTSTREAM_H__
#define __OMLMPIINPUTSTREAM_H__

#include "omlinputstream.h"
#include "omlmpistream.h"

/*-----------------------------------------------------------------*/
/*!
    @class OML_MPIinputstream
    \brief  classe de base pour gerer un flux entrant vers un 
    flux MPI en  Open Math
*/
/*-----------------------------------------------------------------*/
class OML_MPIinputstream : public OML_inputstream,  public OML_MPIstream
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   // ! use the file descriptor fd (return by open for example)
   OML_MPIinputstream() : OML_inputstream(), OML_MPIstream() {};
   OML_MPIinputstream(int src, int tag, MPI_communicator comm = MPI_COMM_WORLD) : OML_inputstream(), OML_MPIstream(src, tag, comm) {};
   virtual ~OML_MPIinputstream() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////
   //! read the xml document
   virtual bool ReadXmlDoc();  
   //! read a line
   virtual bool ReadLine(SCSCP_status* status);

private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   //! set flag of errors to true if value = -1
   template<typename T> void CheckErrors(T value)
   {
    if (value==-1) SetError();
   }
};


#endif /*__OMLMPIINPUTSTREAM_H__*/
