/*-----------------------------------------------------------------*/
/*! 
  \file OMLOMF.h
  \brief Open Math Float object (OMF) definition.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __OMLOMF_H__
#define __OMLOMF_H__

#include "omlinputstream.h"

/*-----------------------------------------------------------------*/
/*!
    @class OML_OMF
    \brief  classe de base pour gerer les flottants Open Math
*/
/*-----------------------------------------------------------------*/
class OML_OMF
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   typedef OML_inputstream::const_OMLchar const_OMLchar;
   typedef OML_inputstream::OMLchar OMLchar;
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   OML_OMF() : m_value(NULL) {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! write an OMF to output stream
   static bool Write(OML_outputstream& stream, double dbl);
   //! write an OMF to output stream
   static bool Write(OML_outputstream& stream, const char *dbl);
   //! read an OMS from input stream
   bool Read(OML_inputstream::iterator& iter);
   //! return the double value
   bool Get(double& integer);
   //! return the value
   const char* Get() const { return m_value; };
   
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////

private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   const_OMLchar m_value;
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // default values
   ////////////////////////////////////////////////////////////////////////////
};


#endif /*__OMLOMF_H__*/
