/*-----------------------------------------------------------------*/
/*! 
  \file omlomobj.cxx
  \brief Open Math Reference object (OMOBJ) implementation.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/

#include <stdio.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include "scscptypes.h"
#include "scscpinternal.h"
#include "omloutputstream.h"
#include "omlinputstream.h"
#include "omlomobj.h"
#include "scscpdebug.h"

/***************************************************************************/
/* macros locales aux fichiers 										   */
/***************************************************************************/
/*! conversion de xmlchar en char */
#define xmlChar2char(x) ((char*)(x))

/*-----------------------------------------------------------------*/
/*! write a begin tag an OMOBJ to output stream
  @param stream (inout) output stream
*/
/*-----------------------------------------------------------------*/
bool OML_OMOBJ::WriteBeginTag(OML_outputstream& stream)
{
  bool bres;
  SCSCP_getdebugger()<<" OML_OMOBJ::WriteBeginTag() - entree\n";
  switch(stream.GetEncoding())
  {
   case OML_eAsciiStream :
        bres = stream.WriteChars("<OMOBJ>");
        break;
        
   case OML_eBinaryStream :
        bres = stream.WriteChars("<OMOBJ>");
        break;
        
   default : bres = false;
        break;
  }
  SCSCP_getdebugger()<<" OML_OMOBJ::WriteBeginTag() - sortie et retourne"<<bres<<"\n";
  return bres;
}

/*-----------------------------------------------------------------*/
/*! write a end tag an OMOBJ to output stream
  @param stream (inout) output stream
*/
/*-----------------------------------------------------------------*/
bool OML_OMOBJ::WriteEndTag(OML_outputstream& stream)
{
  bool bres;
  SCSCP_getdebugger()<<" OML_OMOBJ::WriteEndTag() - entree\n";
  switch(stream.GetEncoding())
  {
   case OML_eAsciiStream :
        bres = stream.WriteChars("</OMOBJ>");
        break;
        
   case OML_eBinaryStream :
        bres = stream.WriteChars("</OMOBJ>");
        break;
        
   default : bres = false;
        break;
  }
  SCSCP_getdebugger()<<" OML_OMOBJ::WriteEndTag() - sortie et retourne"<<bres<<"\n";
  return bres;
}

