/*-----------------------------------------------------------------*/
/*! 
  \file omloutputstream.cxx
  \brief Open Math Output Stream implemntation.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#include <stdio.h>
#include <float.h>
#include "scscptypes.h"
#include "scscpinternal.h"
#include "omloutputstream.h"
#include "omloms.h"
#include "omlomi.h"
#include "omlomstr.h"
#include "omlomr.h"
#include "omlomobj.h"

/*-----------------------------------------------------------------*/
//! write begin tag OMATP
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteBeginTagOMATP()
{
 return WriteChars("<OMATP>");
}

/*-----------------------------------------------------------------*/
//! write end tag OMATP
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteEndTagOMATP()
{
 return WriteChars("</OMATP>");
}

/*-----------------------------------------------------------------*/
//! write begin tag OMATTR
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteBeginTagOMATTR()
{
 return WriteChars("<OMATTR>");
}

/*-----------------------------------------------------------------*/
//! write end tag OMATTR
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteEndTagOMATTR()
{
 return WriteChars("</OMATTR>");
}

/*-----------------------------------------------------------------*/
//! write begin tag OMA
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteBeginTagOMA()
{
 return WriteChars("<OMA>");
}

/*-----------------------------------------------------------------*/
//! write end tag OMA
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteEndTagOMA()
{
 return WriteChars("</OMA>");
}

/*-----------------------------------------------------------------*/
//! write begin tag OMI
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteBeginTagOMI()
{
 return WriteChars("<OMI>");
}

/*-----------------------------------------------------------------*/
//! write end tag OMI
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteEndTagOMI()
{
 return WriteChars("</OMI>");
}

/*-----------------------------------------------------------------*/
/*! write  tag OMF
  @param dbl (in) double
*/
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteTagOMF(double dbl)
{
 char valeur[400];
 sprintf(valeur,"<OMF dec=\"%+-.*E\" />",(int)DBL_DIG+1,dbl);
 return WriteChars(valeur);
}

/*-----------------------------------------------------------------*/
/*! write  tag OMF
  @param dbl (in) double
*/
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteTagOMF(const char* dbl)
{
 char valeur[400];
 sprintf(valeur,"<OMF dec=\"%s\" />",dbl);
 return WriteChars(valeur);
}

/*-----------------------------------------------------------------*/
//! write begin tag OME
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteBeginTagOME()
{
 return WriteChars("<OME>");
}

/*-----------------------------------------------------------------*/
//! write end tag OME
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteEndTagOME()
{
 return WriteChars("</OME>");
}


/*-----------------------------------------------------------------*/
//! write begin tag OMSTR
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteBeginTagOMSTR()
{
 return WriteChars("<OMSTR>");
}

/*-----------------------------------------------------------------*/
//! write end tag OMSTR
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteEndTagOMSTR()
{
 return WriteChars("</OMSTR>");
}

/*-----------------------------------------------------------------*/
//! write an empty OMSTR object
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteBeginEndTagOMSTR()
{
 return WriteChars("<OMSTR/>");
}

/*-----------------------------------------------------------------*/
/*! write an OMR object with a value
 @param string (in) value
*/
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteBeginEndTagOMR(const_OMLchar string)
{
 WriteChars("<OMR xref=\"");
 WriteChars(string);
 return  WriteChars("\" />");
}

/*-----------------------------------------------------------------*/
/*! write an OMV object with a value
 @param string (in) value
*/
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteBeginEndTagOMV(const_OMLchar string)
{
 WriteChars("<OMV name=\"");
 WriteChars(string);
 return  WriteChars("\" />");
}

/*-----------------------------------------------------------------*/
/*! write a pair of OMS-OMI
 @param cdname  (in) cd name of the symbol
 @param name (in) name of the symbol
 @param integer (in) value to write
*/
/*-----------------------------------------------------------------*/
bool OML_outputstream::WritePairOMSOMI(const_OMLchar cdname, const_OMLchar name, size_t integer)
{
 OML_OMS::Write(*this, cdname, name);
 return OML_OMI::Write(*this, integer);
}

/*-----------------------------------------------------------------*/
/*!  write a pair of OMS-OMSTR
 @param cdname  (in) cd name of the symbol
 @param name (in) name of the symbol
 @param string (in) value to write
*/
/*-----------------------------------------------------------------*/
bool OML_outputstream::WritePairOMSOMSTR(const_OMLchar cdname, const_OMLchar name, const_OMLchar string)
{
 OML_OMS::Write(*this, cdname, name);
 return OML_OMSTR::Write(*this, string);
}

/*-----------------------------------------------------------------*/
/*!  write a pair of OMS-OMR
 @param cdname  (in) cd name of the symbol
 @param name (in) name of the symbol
 @param string (in) value to write
*/
/*-----------------------------------------------------------------*/
bool OML_outputstream::WritePairOMSOMR(const_OMLchar cdname, const_OMLchar name, const_OMLchar string)
{
 OML_OMS::Write(*this, cdname, name);
 return OML_OMR::Write(*this, string);
}

/*-----------------------------------------------------------------*/
/*! write characters to stream
 @param data  (in) value to write
*/
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteChars(const_OMLchar data)
{
 printf("%s",data); 
 return hasnoerror();
}

/*-----------------------------------------------------------------*/
/*! write characters to stream
 @param data  (in) value to write
*/
/*-----------------------------------------------------------------*/
OML_outputstream& OML_outputstream::operator<<(const_OMLchar data)
{
 printf("%s",data);
 return *this;
}

/*-----------------------------------------------------------------*/
/*! write characters to stream
 @param data  (in) value to write
*/
/*-----------------------------------------------------------------*/
OML_outputstream& OML_outputstream::operator<<(size_t data)
{
 printf("%llu", (unsigned long long) data);
 return *this;
}

/*-----------------------------------------------------------------*/
/*! write characters to stream
 @param data  (in) value to write
*/
/*-----------------------------------------------------------------*/
OML_outputstream& OML_outputstream::operator<<(int data)
{
 printf("%d", data);
 return *this;
}

/*-----------------------------------------------------------------*/
/*! write characters to stream
 @param data  (in) value to write
*/
/*-----------------------------------------------------------------*/
OML_outputstream& OML_outputstream::operator<<(long long data)
{
 printf("%lld", data);
 return *this;
}

/*-----------------------------------------------------------------*/
/*! write characters to stream
 @param data  (in) value to write
*/
/*-----------------------------------------------------------------*/
OML_outputstream& OML_outputstream::operator<<(long data)
{
 printf("%ld", data);
 return *this;
}


/*-----------------------------------------------------------------*/
/*! write SCSCP start message to the client
*/
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteSCSCPStartMsg()
{
  Flush();
  WriteChars("<?scscp start ?>");
  return Flush();
}

/*-----------------------------------------------------------------*/
/*! write SCSCP end message to the client
*/
/*-----------------------------------------------------------------*/
bool OML_outputstream::WriteSCSCPEndMsg()
{
 Flush();
 WriteChars("<?scscp end ?>");
 return Flush();
}

