/*-----------------------------------------------------------------*/
/*! 
  \file omloutputstream.h
  \brief Open Math Output Stream definition.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __OMLOUTPUTSTREAM_H__
#define __OMLOUTPUTSTREAM_H__

#include "omltypes.h"

/*-----------------------------------------------------------------*/
/*!
    @class OML_outputstream
    \brief  classe de base pour gerer un flux sortant Open Math
*/
/*-----------------------------------------------------------------*/
class OML_outputstream
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   //! type of characters
   typedef char* OMLchar;
   typedef const char* const_OMLchar;

   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   OML_outputstream(OML_eEncoding encode=OML_eAsciiStream) : m_haveerror(false), m_encoder(encode) {};
   virtual ~OML_outputstream() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! write begin tag ATTR
   bool WriteBeginTagOMATTR();
   //! write end tag ATTR
   bool WriteEndTagOMATTR();
   //! write begin tag OMA
   bool WriteBeginTagOMA();
   //! write end tag OMA
   bool WriteEndTagOMA();
   //! write begin tag ATP
   bool WriteBeginTagOMATP();
   //! write end tag ATP
   bool WriteEndTagOMATP();
   //! write begin tag OMI
   bool WriteBeginTagOMI();
   //! write end tag OMI
   bool WriteEndTagOMI();
   //! write tag OMF
   bool WriteTagOMF(double value);
   //! write tag OMF
   bool WriteTagOMF(const char* value);
   //! write tag OMV
   bool WriteBeginEndTagOMV(const_OMLchar string);
   //! write begin tag OME
   bool WriteBeginTagOME();
   //! write end tag OME
   bool WriteEndTagOME();
   //! write begin tag OMSTR
   bool WriteBeginTagOMSTR();
   //! write end tag OMSTR
   bool WriteEndTagOMSTR();
   //! write an empty OMSTR object
   bool WriteBeginEndTagOMSTR();
   //! write an OMR object with a value
   bool WriteBeginEndTagOMR(const_OMLchar string);
   //! write a pair of OMS-OMI
   bool WritePairOMSOMI(const_OMLchar cdname, const_OMLchar name, size_t integer);
   //! write a pair of OMS-OMSTR
   bool WritePairOMSOMSTR(const_OMLchar cdname, const_OMLchar name, const_OMLchar string);
   //! write a pair of OMS-OMR
   bool WritePairOMSOMR(const_OMLchar cdname, const_OMLchar name, const_OMLchar string);
   //! write start message for SCSCP
   bool WriteSCSCPStartMsg();
   //! write end message for SCSCP
   bool WriteSCSCPEndMsg();
   //! get encoding type
   OML_eEncoding GetEncoding() const { return m_encoder; };
   
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////
   //! fush the buffer to stream
   virtual bool Flush() { return true; };
   //! write characters to stream
   virtual bool WriteChars(const_OMLchar data);
   //! write characters to stream
   virtual OML_outputstream& operator<<(const_OMLchar data);
   //! write size_t to stream
   virtual OML_outputstream& operator<<(size_t data);
   //! write long long to stream
   virtual OML_outputstream& operator<<(long long int  data);
   //! write long to stream
   virtual OML_outputstream& operator<<(long int  data);
   //! write int to stream
   virtual OML_outputstream& operator<<(int data);
   
   //! return false on error 
   bool hasnoerror() { return !m_haveerror; };

protected:
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   //! set the flag erro to true
   void SetError() { m_haveerror = true; };
   
private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   bool m_haveerror; //!< =true => errors on write
   OML_eEncoding m_encoder; //!< encoding type
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // default values
   ////////////////////////////////////////////////////////////////////////////
};


#endif /*__OMLOUTPUTSTREAM_H__*/
