/*-----------------------------------------------------------------*/
/*! 
  \file scscp.h
  \brief SCSCP library.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/

#ifndef __SCSCP_H__
#define __SCSCP_H__


/*----------------------------------------------------------------------------------------------*/
/* private part of the library */
/*----------------------------------------------------------------------------------------------*/
#if !defined(__SCSCPINTERNAL_H__)
typedef void* SCSCP_socketclient;
typedef void* SCSCP_socketserver;
typedef void* SCSCP_io;
typedef void* SCSCP_incomingclient;
typedef void* SCSCP_calloptions;
typedef void* SCSCP_returnoptions;
typedef void* SCSCP_xmlnodeptr;
typedef void* SCSCP_xmlattrptr;
#endif /*SCSCP_COMPILELIB*/


/*----------------------------------------------------------------------------------------------*/
/* public part */
/*----------------------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------------------------*/
/* windows specific support */
/*----------------------------------------------------------------------------------------------*/
/* Support for WINDOWS Dll:
   Check if we are inside a SCSCP build, and if so export the functions.
   Otherwise does the same thing as SCSCP */
#if defined (__GNUC__)
#define __SCSCP_DECLSPEC_EXPORT  __declspec(__dllexport__)
#define __SCSCP_DECLSPEC_IMPORT  __declspec(__dllimport__)
#endif
#if defined (_MSC_VER) || defined (__BORLANDC__)
#define __SCSCP_DECLSPEC_EXPORT  __declspec(dllexport)
#define __SCSCP_DECLSPEC_IMPORT  __declspec(dllimport)
#endif
#ifdef __WATCOMC__
#define __SCSCP_DECLSPEC_EXPORT  __export
#define __SCSCP_DECLSPEC_IMPORT  __import
#endif
#ifdef __IBMC__
#define __SCSCP_DECLSPEC_EXPORT  _Export
#define __SCSCP_DECLSPEC_IMPORT  _Import
#endif

#if __SCSCP_LIBSCSCP_DLL
/* compile as a dll */
#if defined(__SCSCP_WITHIN_SCSCP)
#define __SCSCP_DECLSPEC __SCSCP_DECLSPEC_EXPORT
#else
#define __SCSCP_DECLSPEC __SCSCP_DECLSPEC_IMPORT
#endif
#else
/* other cases */
#define __SCSCP_DECLSPEC 
#endif /*__SCSCP_DECLSPEC*/

/*----------------------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------------------------*/
#if defined (__cplusplus)
extern "C" {
#endif /*defined (__cplusplus)*/

#include "scscptypes.h"

/*----------------------------------------------------------------------------------------------*/
/* definition of the SCSCP library version */
/*! version : major number of SCSCP library */
#define SCSCP_VERSION_MAJOR 0
/*! version : minor number of SCSCP library */
#define SCSCP_VERSION_MINOR 3
/*! version : patch number of SCSCP library */
#define SCSCP_VERSION_PATCH 2

/*----------------------------------------------------------------------------------------------*/
/* definition of the standard version number of  SCSCP  */
/*! definition of the standard version number of  SCSCP  : version 1.1 */
#define SCSCP_PROTOCOL_VERSION_1_1 "1.1"
/*! definition of the standard version number of  SCSCP  : version 1.2 */
#define SCSCP_PROTOCOL_VERSION_1_2 "1.2"

/*----------------------------------------------------------------------------------------------*/
/*! default value on which port should listen the SCSCP server */
#define SCSCP_PROTOCOL_DEFAULTPORT 26133

/*----------------------------------------------------------------------------------------------*/
/* functions for the SCSCP client and server  */
/*----------------------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------------------------*/
/* defines and functions for status object */
/*----------------------------------------------------------------------------------------------*/

/*! default status value to ignore returned value */
#define SCSCP_STATUS_IGNORE NULL

/*!possible value for SCSCP_status */
/*! SCSCP library error code : no error */
#define SCSCP_STATUS_OK 0
/*! SCSCP library error code : The object client passed to the function is NULL */
#define SCSCP_STATUS_CLIENTOBJECTNULL (-1)
/*! SCSCP library error code : Not enough memory */
#define SCSCP_STATUS_NOMEM (-2)
/*!  errno is set to a system error */
#define SCSCP_STATUS_ERRNO (-3)
/*! SCSCP library error code : The call options passed to the function is NULL */
#define SCSCP_STATUS_CALLOPTIONSOBJECTNULL (-4)
/*! SCSCP library error code : The interrupt message '<?scscp cancel ?> was received */
#define SCSCP_STATUS_RECVCANCEL (-8)
/*! SCSCP library error code : The interrupt message '<?scscp quit ?> was received */
#define SCSCP_STATUS_RECVQUIT (-9)
/*! SCSCP library error code : The return options passed to the function is NULL */
#define SCSCP_STATUS_RETURNOPTIONSOBJECTNULL (-5)
/*! SCSCP library error code : The object server passed to the function is NULL */
#define SCSCP_STATUS_SERVEROBJECTNULL (-6)
/*! SCSCP library error code : The object stream passed to the function is NULL */
#define SCSCP_STATUS_STREAMOBJECTNULL (-7)
/*! SCSCP library error code : The runtime usage isn't available in the procedure return message */
#define SCSCP_STATUS_USAGEUNKNOWNRUNTIME (-20)
/*! SCSCP library error code : The memory usage isn't available in the procedure return message */
#define SCSCP_STATUS_USAGEUNKNOWNMEM (-21)
/*! SCSCP library error code : The runtime limit usage isn't available in the procedure call message */
#define SCSCP_STATUS_USAGEUNKNOWNRUNTIMELIMIT (-22)
/*! SCSCP library error code : The min memory isn't available in the procedure call message */
#define SCSCP_STATUS_USAGEUNKNOWNMINMEMORY (-23)
/*! SCSCP library error code : The max memory isn't available in the procedure call message */
#define SCSCP_STATUS_USAGEUNKNOWNMAXMEMORY (-24)
/*! SCSCP library error code : The debug level isn't available in the procedure call message */
#define SCSCP_STATUS_USAGEUNKNOWNDEBUGLEVEL (-25)
/*! SCSCP library error code : The return type isn't available in the procedure call message */
#define SCSCP_STATUS_USAGEUNKNOWNRETURNTYPE (-26)
/*! SCSCP library error code : ther version negotiation fails */
#define SCSCP_STATUS_VERSIONNEGOTIATIONFAILED (-30)


/*! copy the content of src to dst */
__SCSCP_DECLSPEC void SCSCP_status_copy(SCSCP_status* dst, const SCSCP_status *src);
/*! return 0 if status indicates an error */
__SCSCP_DECLSPEC int SCSCP_status_isok(const SCSCP_status* status);
/*! return the numerical value of the status */
#define SCSCP_status_is(status) (*(status))

/*----------------------------------------------------------------------------------------------*/
/* defines and functions for procedure call options object */
/*----------------------------------------------------------------------------------------------*/
/*! default procedure call option to send to server */
#define SCSCP_CALLOPTIONS_DEFAULT NULL
/*! default procedure call option to ignore returned value  */
#define SCSCP_CALLOPTIONS_IGNORE NULL

/*! initialize the call options and set status if an error occurs */
__SCSCP_DECLSPEC int SCSCP_co_init(SCSCP_calloptions* options, SCSCP_status* status);
/*! clear the call options previously initialized by SCSCP_co_init */
__SCSCP_DECLSPEC int SCSCP_co_clear(SCSCP_calloptions* options, SCSCP_status* status);
/*! set the procedure call ID (buffer can't be released before calling SCSCP_co_clear) */
__SCSCP_DECLSPEC int SCSCP_co_set_callid(SCSCP_calloptions* options, const char *buffer, SCSCP_status* status);
/*! get the procedure call ID */
__SCSCP_DECLSPEC int SCSCP_co_get_callid(SCSCP_calloptions* options, const char **buffer, SCSCP_status* status);
/*! set the runtime limit */
__SCSCP_DECLSPEC int SCSCP_co_set_runtimelimit(SCSCP_calloptions* options, size_t time, SCSCP_status* status);
/*! get the runtime limit */
__SCSCP_DECLSPEC int SCSCP_co_get_runtimelimit(SCSCP_calloptions* options, size_t* time, SCSCP_status* status);
/*! set the minimum memory */
__SCSCP_DECLSPEC int SCSCP_co_set_minmemory(SCSCP_calloptions* options, size_t memsize, SCSCP_status* status);
/*! get the minimum memory */
__SCSCP_DECLSPEC int SCSCP_co_get_minmemory(SCSCP_calloptions* options, size_t* memsize, SCSCP_status* status);
/*! set the maximum memory */
__SCSCP_DECLSPEC int SCSCP_co_set_maxmemory(SCSCP_calloptions* options, size_t memsize, SCSCP_status* status);
/*! get the maximum memory */
__SCSCP_DECLSPEC int SCSCP_co_get_maxmemory(SCSCP_calloptions* options, size_t* memsize, SCSCP_status* status);
/*! set the debug level */
__SCSCP_DECLSPEC int SCSCP_co_set_debuglevel(SCSCP_calloptions* options, int debuglevel, SCSCP_status* status);
/*! get the debug level */
__SCSCP_DECLSPEC int SCSCP_co_get_debuglevel(SCSCP_calloptions* options, int* debuglevel, SCSCP_status* status);
/*! set the return type */
__SCSCP_DECLSPEC int SCSCP_co_set_returntype(SCSCP_calloptions* options, SCSCP_option_return returntype, SCSCP_status* status);
/*! get the return type */
__SCSCP_DECLSPEC int SCSCP_co_get_returntype(SCSCP_calloptions* options, SCSCP_option_return* returntype, SCSCP_status* status);

/*----------------------------------------------------------------------------------------------*/
/* defines and functions for procedure return options object */
/*----------------------------------------------------------------------------------------------*/
/*! default procedure return option to ignore returned value  */
#define SCSCP_RETURNOPTIONS_IGNORE NULL
/*! default procedure return option to send to client  */
#define SCSCP_RETURNOPTIONS_DEFAULT NULL

/*! initialize the return options and set status if an error occurs */
__SCSCP_DECLSPEC int SCSCP_ro_init(SCSCP_returnoptions* options, SCSCP_status* status);
/*! clear the return options previously initialized by SCSCP_ro_init */
__SCSCP_DECLSPEC int SCSCP_ro_clear(SCSCP_returnoptions* options, SCSCP_status* status);
/*! set the procedure call ID */
__SCSCP_DECLSPEC int SCSCP_ro_set_callid(SCSCP_returnoptions* options, const char *buffer, SCSCP_status* status);
/*! get the procedure call ID */
__SCSCP_DECLSPEC int SCSCP_ro_get_callid(SCSCP_returnoptions* options, const char **buffer, SCSCP_status* status);
/*! set the runtime usage */
__SCSCP_DECLSPEC int SCSCP_ro_set_runtime(SCSCP_returnoptions* options, size_t time, SCSCP_status* status);
/*! get the runtime usage */
__SCSCP_DECLSPEC int SCSCP_ro_get_runtime(SCSCP_returnoptions* options, size_t* time, SCSCP_status* status);
/*! set the memory usage */
__SCSCP_DECLSPEC int SCSCP_ro_set_memory(SCSCP_returnoptions* options, size_t mem, SCSCP_status* status);
/*! get the memory usage */
__SCSCP_DECLSPEC int SCSCP_ro_get_memory(SCSCP_returnoptions* options, size_t* mem, SCSCP_status* status);

/*----------------------------------------------------------------------------------------------*/
/* functions for the SCSCP_xmlnodeptr/SCSCP_xmlattr only  */
/*----------------------------------------------------------------------------------------------*/
/*! return the next node after the current xml node  */
__SCSCP_DECLSPEC SCSCP_xmlnodeptr SCSCP_xmlnode_getnext(SCSCP_xmlnodeptr curnode);
/*! return the name after the current xml node  */
__SCSCP_DECLSPEC const char* SCSCP_xmlnode_getname(SCSCP_xmlnodeptr curnode);
/*! return the first child node in the current xml node  */
__SCSCP_DECLSPEC SCSCP_xmlnodeptr SCSCP_xmlnode_getchild(SCSCP_xmlnodeptr curnode);
/*! read the content of the current node in the current xml node */
__SCSCP_DECLSPEC const char * SCSCP_xmlnode_getcontent(SCSCP_xmlnodeptr node);
/*! read the first attribute of the current node  */
__SCSCP_DECLSPEC SCSCP_xmlattrptr SCSCP_xmlnode_getattr(SCSCP_xmlnodeptr curnode);
/*! return the next attribute after attr */
__SCSCP_DECLSPEC SCSCP_xmlattrptr SCSCP_xmlattr_getnext(SCSCP_xmlattrptr attr);
/*! get the content of the attribute */
__SCSCP_DECLSPEC int SCSCP_xmlattr_getvalue(SCSCP_xmlattrptr attr, const char **name, const char **value);

/*----------------------------------------------------------------------------------------------*/
/* functions for the SCSCP low I/O   */
/*----------------------------------------------------------------------------------------------*/
/*! write the openmath expression directly to the SCSCP client or server 
    (should only be called fron function SCSCP_xx_xxxcallback) */
__SCSCP_DECLSPEC int SCSCP_io_write(SCSCP_io* stream, const char *buffer, SCSCP_status* status);


/*----------------------------------------------------------------------------------------------*/
/* functions for the SCSCP client only  */
/*----------------------------------------------------------------------------------------------*/
/*! initialize the object client and set status if an error occurs */
__SCSCP_DECLSPEC int SCSCP_sc_init(SCSCP_socketclient* client, SCSCP_status* status, ...); 
/*! clear the object client previously initialized by SCSCP_sc_init */
__SCSCP_DECLSPEC int SCSCP_sc_clear(SCSCP_socketclient* client, SCSCP_status* status); 

/*! connect the client to a server on the specified host and port */
__SCSCP_DECLSPEC int SCSCP_sc_connect(SCSCP_socketclient* client, const char *machine, int port, SCSCP_status* status); 
/*! close the connection of the client to a server previously opened by SCSCP_sc_connect */
__SCSCP_DECLSPEC int SCSCP_sc_close(SCSCP_socketclient* client, SCSCP_status* status); 

/*! send a procedure call with a openmath expression as arguments to the SCSCP server */
__SCSCP_DECLSPEC int SCSCP_sc_callsendstr(SCSCP_socketclient* client, SCSCP_calloptions* options, 
                          const char *openmathbuffer, SCSCP_status* status);
/*! send a procedure call to the SCSCP server. The arguments  are written by callbackwriteargs */
__SCSCP_DECLSPEC int SCSCP_sc_callsendcallback(SCSCP_socketclient* client, SCSCP_calloptions* options, 
                          int (*callbackwriteargs)(SCSCP_io* stream, void *param, SCSCP_status* status),
                          void *param, SCSCP_status* status);
/*! read the attribute of the answer for the procedure call and the type of the answer message */
__SCSCP_DECLSPEC int SCSCP_sc_callrecvheader(SCSCP_socketclient* client, SCSCP_returnoptions* options,
                            SCSCP_msgtype *msgtype, SCSCP_status* status);
                            

/* for "procedure terminated" message */
/*! read the content of a "procedure terminated message" 
    and the error message is stored in  errortype and messagebuffer  */
__SCSCP_DECLSPEC int SCSCP_sc_callrecvterminated(SCSCP_socketclient* client, SCSCP_error* errortype, char **messagebuffer, SCSCP_status* status); 

/* for "procedure completed" message */
/*! read the content of a "procedure completed message" and store it in openmathbuffer */
__SCSCP_DECLSPEC int SCSCP_sc_callrecvcompleted(SCSCP_socketclient* client, char **openmathbuffer, SCSCP_status* status); 

/*! read the answer of the procedure call and store it in a string buffer */
__SCSCP_DECLSPEC int SCSCP_sc_callrecvstr(SCSCP_socketclient* client, SCSCP_returnoptions* options,
                         SCSCP_msgtype *msgtype, char **openmathbuffer, SCSCP_status* status); 

/*! return the first node in the current location of the openmath message */
__SCSCP_DECLSPEC SCSCP_xmlnodeptr SCSCP_sc_getxmlnode(SCSCP_socketclient *client, SCSCP_status* status);


/*----------------------------------------------------------------------------------------------*/
/* functions for the SCSCP server only  */
/*----------------------------------------------------------------------------------------------*/
/*! initialize the object server and set status if an error occurs */
__SCSCP_DECLSPEC int SCSCP_ss_init(SCSCP_socketserver* server, SCSCP_status* status, const char *servicename, 
                  const char *serviceversion, const char *serviceid, ...); 
/*! clear the object server previously initialized by SCSCP_ss_init */
__SCSCP_DECLSPEC int SCSCP_ss_clear(SCSCP_socketserver* server, SCSCP_status* status); 

/*! the object server listen on the specified port */
__SCSCP_DECLSPEC int SCSCP_ss_listen(SCSCP_socketserver* server, int port, int firstavailable, SCSCP_status* status); 
/*! the object server terminates to listen */
__SCSCP_DECLSPEC int SCSCP_ss_close(SCSCP_socketserver* server, SCSCP_status* status);  

/*! the object accepts an incoming connection and returns a incoming client connection */
__SCSCP_DECLSPEC int SCSCP_ss_acceptclient(SCSCP_socketserver* server, SCSCP_incomingclient* incomingclient, SCSCP_status* status); 

/*----------------------------------------------------------------------------------------------*/
/* functions for the SCSCP incoming connection only (server only)  */
/*----------------------------------------------------------------------------------------------*/
/*! receive the "procedure call" message from the client :
  read the attribute of the incoming procedure call and the type of the incoming message 
  store the arguments in the openmathbuffer */
__SCSCP_DECLSPEC int SCSCP_ss_callrecvstr(SCSCP_incomingclient* incomingclient, SCSCP_calloptions* options,
                         SCSCP_msgtype *msgtype, char **openmathbuffer, SCSCP_status* status); 
/*! receive the "procedure call" message  : 
    read the attribute of the incoming procedure call and the type of the incoming message */
__SCSCP_DECLSPEC int SCSCP_ss_callrecvheader(SCSCP_incomingclient* incomingclient, SCSCP_calloptions* options,
                            SCSCP_msgtype *msgtype, SCSCP_status* status);

/*! the server sends the "procedure terminated" message with an error type and a text message to the client */
__SCSCP_DECLSPEC int SCSCP_ss_sendterminatedstr(SCSCP_incomingclient* incomingclient, SCSCP_returnoptions* options,  SCSCP_error errortype,
                                const char *messagebuffer, SCSCP_status* status);

/*! the server sends the "procedure completed" message  with a openmath expression as arguments  to the client */
__SCSCP_DECLSPEC int SCSCP_ss_sendcompletedstr(SCSCP_incomingclient* incomingclient, SCSCP_returnoptions* options, 
                              const char *openmathbuffer, SCSCP_status* status); 
/*! the server sends the "procedure completed" message to the client. The arguments  are written by callbackwriteargs */
__SCSCP_DECLSPEC int SCSCP_ss_sendcompletedcallback(SCSCP_incomingclient* incomingclient, SCSCP_returnoptions* options, 
                          int (*callbackwriteargs)(SCSCP_io* stream, void *param, SCSCP_status* status),
                          void *param, SCSCP_status* status);

/*! terminate an incoming connection */
__SCSCP_DECLSPEC int SCSCP_ss_closeincoming(SCSCP_incomingclient* client, SCSCP_status* status);  

/*! return the first node in the current location of the openmath message */
__SCSCP_DECLSPEC SCSCP_xmlnodeptr SCSCP_ss_getxmlnode(SCSCP_incomingclient* client, SCSCP_status* status);

#if defined (__cplusplus)
}
#endif /*defined (__cplusplus)*/

#endif /*__SCSCP_H__*/
