/*-----------------------------------------------------------------*/
/*! 
  \file scscpcallid.cxx
  \brief SCSCP call ID implementation
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/

#include <stdio.h>
#include "scscptypes.h"
#include "scscpinternal.h"
#include "scscpcallid.h"
#include "scscptags.h"

   
/*-----------------------------------------------------------------*/
/*! write the object to output stream
  @param stream (inout) output stream
*/
/*-----------------------------------------------------------------*/
bool SCSCP_call_ID::Write(OML_outputstream& stream) const
{
 return stream.WritePairOMSOMSTR(SCSCP_tags::cdname, SCSCP_tags::call_ID , m_ID);
}
  
/*-----------------------------------------------------------------*/
/*! read the object from th input stream
  @param stream (inout) input stream
  @param iter (inout) iterator on the input stream
*/
/*-----------------------------------------------------------------*/
bool SCSCP_call_ID::Read(OML_inputstream& stream, OML_inputstream::iterator& iter)
{
 return stream.ReadPairOMSOMSTR(iter, SCSCP_tags::cdname, SCSCP_tags::call_ID , m_ID);
}
  
/*-----------------------------------------------------------------*/
/*! return the string buffer of the call id
  @param callid (in) call id
*/
/*-----------------------------------------------------------------*/
const_SCSCP_string SCSCP_call_ID_Get(const SCSCP_call_ID *callid)
{
 return callid->m_ID;
}
