/*-----------------------------------------------------------------*/
/*! 
  \file scscpfileserver.h
  \brief SCSCP server with file descriptor
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __SCSCPFILESERVER_H__
#define __SCSCPFILESERVER_H__

#include "scscpserver.h"

/*-----------------------------------------------------------------*/
/*!
    @class SCSCP_Fileserver
    \brief  classe de base pour gerer un serveur SCSCP
*/
/*-----------------------------------------------------------------*/
class SCSCP_Fileserver : public SCSCP_server
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_Fileserver() : SCSCP_server(), m_fd(-1) {};
   virtual ~SCSCP_Fileserver() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! Listen on a specified port  
   bool Listen(int port=26133);
   //! Accept connection (must be called after Listen)
   bool Accept(OML_Fileinputstream& stream);
   //! Accept connection and return a socket descriptor (must be called after Listen)
   bool Accept(int& newsocket);
   //! close the connexion to the server 
   void Close();
   //! close a socket
   void Close(int socket);
   
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////
   //! get the socket
   int GetSocket() const { return m_fd; };


private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   int m_fd; //!< socket descriptor

   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   //! wait for the connection information message
   bool WaitConnectionInformationMessage();
   //! read the Connection information Message from the client
   bool ReadCIM(int socket, const char **supportedversion);
   //! write the Connection information Message to the client
   bool WriteCIM(int socket);
   //! write the CIM version to the client
   bool WriteCIMVersion(int socket, const char *supportedversion);
   //! read a line from the client
   bool ReadLine(int socket);
};


#endif /*__SCSCPFILESERVER_H__*/
