/*-----------------------------------------------------------------*/
/*! 
  \file scscpinternal.h
  \brief SCSCP internal functions
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#if !defined(__SCSCPINTERNAL_H__) 
#define __SCSCPINTERNAL_H__

#include <stdlib.h>

#if !defined(__UTIL_H__)
inline void *operator new( size_t s ) { void* p=(void*)malloc(s); return p; }
/*! overload the operator delete */
inline void operator delete (void* p) { free(p); }
/*! overload the operator new[] */
inline void *operator new[] ( size_t s ) { void* p=(void*)malloc(s); return p; }
/*! overload the operator delete[] */
inline void operator delete[]  (void* p) { free(p); }
#endif 

bool SCSCP_initsocket();
/* macros pour compatibilite des sockets entre unix et windows */
#if !HAVE_WINDOWS_H || !defined(INVALID_SOCKET)
#define INVALID_SOCKET (-1) 
#define SOCKET_ERROR (-1) 
#define closesocket(sock) close(sock)
#endif

/*! allocate memory of ncount bytes (similar to malloc) */
void* SCSCP_malloc(size_t ncount, SCSCP_status* status);
/*! duplicate a string (similar to strdup) */
char* SCSCP_strdup(const char*, SCSCP_status* status);

class SCSCP_Fileclient;
class SCSCP_Fileserver;
class SCSCP_procedurecalloptions;
class SCSCP_optionsprocedurereturn;
typedef SCSCP_Fileclient* SCSCP_socketclient;
typedef SCSCP_Fileserver* SCSCP_socketserver;
typedef SCSCP_Fileclient* SCSCP_incomingclient;
typedef SCSCP_Fileclient* SCSCP_io;
typedef SCSCP_procedurecalloptions* SCSCP_calloptions;
typedef SCSCP_optionsprocedurereturn* SCSCP_returnoptions;

int SCSCP_sc_checkNULL(SCSCP_socketclient* client, SCSCP_status* status);
int SCSCP_ss_checkNULL(SCSCP_Fileserver* server, SCSCP_status* status);

char* SCSCP_findversionstring(char *c, const char *versionname);

#endif /*__SCSCPINTERNAL_H__*/
