/*-----------------------------------------------------------------*/
/*! 
  \file scscpprocedurecall.cxx
  \brief SCSCP message : procedure call.
   class SCSCP_procedurecalloptions, SCSCP_procedurecall
   types SCSCP_option_return
 
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr

  @bug M. GASTINEAU 12/09/08 : update for SCSCP 1.2
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
 This library is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
*/
/*-----------------------------------------------------------------*/



#include <stdio.h>
#if HAVE_STRING_H
#include <string.h>
#endif
#include <libxml/parser.h>
#include <libxml/tree.h>
#include "scscptypes.h"
#include "scscpinternal.h"
#include "scscpprocedurecall.h"
#include "scscpdebug.h"
#include "scscpfileclient.h"
#include "scscptags.h"
#include "omlomobj.h"
#include "omloms.h"
#define __SCSCP_WITHIN_SCSCP
#include "scscp.h"


/*-----------------------------------------------------------------*/
/*!
    write the object to the stream
   @param stream (inout) output stream
*/
/*-----------------------------------------------------------------*/
bool SCSCP_procedurecalloptions::Write(OML_outputstream& stream) const
{
 SCSCP_getdebugger()<<"\nSCSCP_procedurecalloptions::Write() - entree\n";
 
 stream.WriteBeginTagOMATP();
 //write call id
 if (m_set_callid) GetCallID().Write(stream);
 //write runtime limit
 if (m_set_runtime_limit) stream.WritePairOMSOMI(SCSCP_tags::cdname, SCSCP_tags::option_runtime, GetRuntimeLimit());
 //write min memory
 if (m_set_minimal_memory) stream.WritePairOMSOMI(SCSCP_tags::cdname, SCSCP_tags::option_min_memory, GetMinimalMemory());
 //write max memory
 if (m_set_maximal_memory) stream.WritePairOMSOMI(SCSCP_tags::cdname, SCSCP_tags::option_max_memory, GetMaximalMemory());
 //write debug level
 if (m_set_minimal_memory) stream.WritePairOMSOMI(SCSCP_tags::cdname, SCSCP_tags::option_debuglevel, GetDebugLevel());
 //write return type
 if (m_set_returntype) 
 {
  const char* returntype = NULL;
  if (GetReturnType()==SCSCP_option_return_object) returntype = SCSCP_tags::option_return_object;
  if (GetReturnType()==SCSCP_option_return_cookie) returntype = SCSCP_tags::option_return_cookie;
  if (GetReturnType()==SCSCP_option_return_nothing) returntype = SCSCP_tags::option_return_nothing;

  stream.WritePairOMSOMSTR(SCSCP_tags::cdname, returntype, NULL);
 }
 bool res = stream.WriteEndTagOMATP();
 
 SCSCP_getdebugger()<<"\nSCSCP_procedurecalloptions::Write() - sortie et retourne "<<res<<"\n";
 return res;
}


/*-----------------------------------------------------------------*/
//! constructor
/*-----------------------------------------------------------------*/
SCSCP_procedurecalloptions::SCSCP_procedurecalloptions()
                          :  m_set_callid(false), m_set_returntype(false), 
                             m_set_runtime_limit(false), m_set_minimal_memory(false),
                             m_set_maximal_memory(false), m_set_debuglevel(false)
{
}

/*-----------------------------------------------------------------*/
/*! read the object from the stream
   @param stream (inout) output stream
   @param iter (inout) iterator on the stream

  @bug M. GASTINEAU 12/09/08 : update for SCSCP 1.2
*/
/*-----------------------------------------------------------------*/
bool SCSCP_procedurecalloptions::Read(OML_inputstream& stream, OML_inputstream::iterator& iter)
{
 bool res = true;
 bool bresfinal = true;
 OML_inputstream::iterator iterend;
 SCSCP_getdebugger()<<"\nSCSCP_procedurecalloptions::Read() - entree\n";
 
 stream.ReadTagOMATTR(iter);
 iter = stream.GoToChild(iter);
 bresfinal = res = stream.ReadTagOMATP(iter);
 iterend = iter;
 /* try to read all attributes */
 if (res)
 {
  OML_inputstream::iterator iterchild = iter->children;
  res  = m_callid.Read(stream, iterchild);
  iterchild = iter->children;
  SCSCP_time timer;
  res = stream.ReadPairOMSOMI(iterchild, SCSCP_tags::cdname, SCSCP_tags::option_runtime, timer); 
  if (res) SetRuntimeLimit(timer);
  iterchild = iter->children;
  SCSCP_memory mem;
  res = stream.ReadPairOMSOMI(iterchild, SCSCP_tags::cdname, SCSCP_tags::option_min_memory, mem); 
  if (res) SetMinimalMemory(mem);
  iterchild = iter->children;
  res = stream.ReadPairOMSOMI(iterchild, SCSCP_tags::cdname, SCSCP_tags::option_max_memory, mem); 
  if (res) SetMaximalMemory(mem);
  SCSCP_debuglevel debuglevel;
  iterchild = iter->children;
  res = stream.ReadPairOMSOMI(iterchild, SCSCP_tags::cdname, SCSCP_tags::option_debuglevel, debuglevel); 
  if (res) SetDebugLevel(debuglevel);
  
   iterchild = iter->children;
   OML_inputstream::iterator iterchild1 = iterchild;
   OML_inputstream::iterator iterchild2 = iterchild;
   OML_inputstream::iterator iterchild3 = iterchild;
   if (stream.ReadOMS(iterchild1, SCSCP_tags::cdname, SCSCP_tags::option_return_object)) 
   { SetReturnType(SCSCP_option_return_object);  }
   else  if (stream.ReadOMS(iterchild2, SCSCP_tags::cdname, SCSCP_tags::option_return_cookie)) 
   { SetReturnType(SCSCP_option_return_cookie);  }
   else  if (stream.ReadOMS(iterchild3, SCSCP_tags::cdname, SCSCP_tags::option_return_nothing)) 
   { SetReturnType(SCSCP_option_return_nothing); }
   else { bresfinal = res = false; }
 }
 
 iter = iterend;

 Debug();
 SCSCP_getdebugger()<<"\nSCSCP_procedurecalloptions::Read() - sortie et retourne "<<bresfinal<<"\n";
 return bresfinal;
}

/*-----------------------------------------------------------------*/
/*! Display debug information about SCSCP_procedurecalloptions
*/
/*-----------------------------------------------------------------*/
void SCSCP_procedurecalloptions::Debug() const
{
 SCSCP_getdebugger()<<"SCSCP_procedurecalloptions object\n";
 if (GetReturnType()==SCSCP_option_return_object) SCSCP_getdebugger()<<"returntype="<<SCSCP_tags::option_return_object<<"\n";
 if (GetReturnType()==SCSCP_option_return_cookie) SCSCP_getdebugger()<<"returntype="<<SCSCP_tags::option_return_cookie<<"\n";
 if (GetReturnType()==SCSCP_option_return_nothing) SCSCP_getdebugger()<<"returntype="<<SCSCP_tags::option_return_nothing<<"\n";
 SCSCP_getdebugger()<<"end of SCSCP_procedurecalloptions object\n";
}

/*-----------------------------------------------------------------*/
/*! checks if options is null and returns 0 on error
  @param options (in) object to be checked
  @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
static int SCSCP_co_checkNULL(SCSCP_calloptions* options, SCSCP_status* status)
{
   int res = 1;
   
   if (options==NULL || *options==NULL)
   {
     SCSCP_status_seterror(status,  SCSCP_STATUS_CALLOPTIONSOBJECTNULL);
     res = 0;
   }
   return res;
}

/*-----------------------------------------------------------------*/
/*! initialize the object options and set status if an error occurs
   @param options (out) options
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_init(SCSCP_calloptions* options, SCSCP_status* status)
{
   int res = 1;
   
   SCSCP_status_seterror(status, SCSCP_STATUS_OK);
   if (options==NULL)
   {
     SCSCP_status_seterror(status,  SCSCP_STATUS_CALLOPTIONSOBJECTNULL);
     res = 0;
   }
   else
   {
    *options = new SCSCP_procedurecalloptions;
    if (*options==NULL) 
    {
     SCSCP_status_seterror(status, SCSCP_STATUS_NOMEM);
     res = 0;
    }
   }
   return res;
}

/*-----------------------------------------------------------------*/
/*!  clear the object options previously initialized by SCSCP_co_init
   @param options (inout) options
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_clear(SCSCP_calloptions* options, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res) 
   {
    SCSCP_status_seterror(status, SCSCP_STATUS_OK);
    delete (*options);
   }
   return res;
}

/*-----------------------------------------------------------------*/
/*! set the procedure call ID (buffer can't be released before calling SCSCP_co_clear)
   @param options (inout) options
   @param buffer (in) string buffer
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_set_callid(SCSCP_calloptions* options, const char *buffer, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res)  (*options)->SetCallID(SCSCP_call_ID(buffer));
   return res;
}

/*-----------------------------------------------------------------*/
/*! get the procedure call ID 
   @param options (inout) options
   @param buffer (out) string buffer
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_get_callid(SCSCP_calloptions* options, const char **buffer, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res)  *buffer = SCSCP_call_ID_Get(&((*options)->GetCallID()));
   return res;
}
/*-----------------------------------------------------------------*/
/*! set the runtime limit
   @param options (inout) options
   @param time (in) runtime limit
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_set_runtimelimit(SCSCP_calloptions* options, size_t time, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res)  (*options)->SetRuntimeLimit(time);
   return res;
}

/*-----------------------------------------------------------------*/
/*! get the runtime limit
  if the runtime limit is unknown => it returns 0 
     and status is set to  SCSCP_STATUS_USAGEUNKNOWNRUNTIMELIMIT
   @param options (inout) options
   @param time (out) runtime limit
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_get_runtimelimit(SCSCP_calloptions* options, size_t* time, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res) 
   {
    res = (*options)->IsRuntimeLimitValid();
    if (!res) SCSCP_status_seterror(status, SCSCP_STATUS_USAGEUNKNOWNRUNTIMELIMIT);
    *time = (*options)->GetRuntimeLimit();
   }
   return res;
}


/*-----------------------------------------------------------------*/
/*! set the minimal memory
   @param options (inout) options
   @param memsize (in) minimal memory required
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_set_minmemory(SCSCP_calloptions* options, size_t memsize, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res)  (*options)->SetMinimalMemory(memsize);
   return res;
}

/*-----------------------------------------------------------------*/
/*! get the minimal memory
  if the minimal memory is unknown => it returns 0 
     and status is set to  SCSCP_STATUS_USAGEUNKNOWNMINMEMORY
   @param options (inout) options
   @param memsize (out) minimal memory required
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_get_minmemory(SCSCP_calloptions* options, size_t* memsize, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res) 
   {
    res = (*options)->IsMinimalMemoryValid();
    if (!res) SCSCP_status_seterror(status, SCSCP_STATUS_USAGEUNKNOWNMINMEMORY);
    *memsize = (*options)->GetMinimalMemory();
   }
   return res;
}

/*-----------------------------------------------------------------*/
/*! set the maximal memory
   @param options (inout) options
   @param memsize (in) maximal memory required
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_set_maxmemory(SCSCP_calloptions* options, size_t memsize, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res)  (*options)->SetMaximalMemory(memsize);
   return res;
}

/*-----------------------------------------------------------------*/
/*! get the maximal memory
  if the minimal memory is unknown => it returns 0 
     and status is set to  SCSCP_STATUS_USAGEUNKNOWNMAXMEMORY
   @param options (inout) options
   @param memsize (out) maximal memory required
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_get_maxmemory(SCSCP_calloptions* options, size_t* memsize, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res) 
   {
    res = (*options)->IsMaximalMemoryValid();
    if (!res) SCSCP_status_seterror(status, SCSCP_STATUS_USAGEUNKNOWNMAXMEMORY);
    *memsize = (*options)->GetMaximalMemory();
   }
   return res;
}


/*-----------------------------------------------------------------*/
/*! set the return type
   @param options (inout) options
   @param returntype (in) return type
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_set_returntype(SCSCP_calloptions* options, SCSCP_option_return returntype, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res)  (*options)->SetReturnType(returntype);
   return res;
}

/*-----------------------------------------------------------------*/
/*! get the return type
  if the minimal memory is unknown => it returns 0 
     and status is set to  SCSCP_STATUS_USAGEUNKNOWNRETURNTYPE
   @param options (inout) options
   @param returntype (out) return type
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_get_returntype(SCSCP_calloptions* options, SCSCP_option_return* returntype, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res) 
   {
    res = (*options)->IsReturnTypeValid();
    if (!res) SCSCP_status_seterror(status, SCSCP_STATUS_USAGEUNKNOWNRETURNTYPE);
    *returntype = (*options)->GetReturnType();
   }
   return res;
}


/*-----------------------------------------------------------------*/
/*! set the debug level
   @param options (inout) options
   @param debuglevel (in) debug level
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_set_debuglevel(SCSCP_calloptions* options, int debuglevel, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res)  (*options)->SetDebugLevel(debuglevel);
   return res;
}

/*-----------------------------------------------------------------*/
/*! get the debug level
  if the minimal memory is unknown => it returns 0 
     and status is set to  SCSCP_STATUS_USAGEUNKNOWNDEBUGLEVEL
   @param options (inout) options
   @param debuglevel (out) debug level
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_co_get_debuglevel(SCSCP_calloptions* options, int* debuglevel, SCSCP_status* status)
{
   int res = SCSCP_co_checkNULL(options, status);
   if (res) 
   {
    res = (*options)->IsDebugLevelValid();
    if (!res) SCSCP_status_seterror(status, SCSCP_STATUS_USAGEUNKNOWNDEBUGLEVEL);
    *debuglevel = (*options)->GetDebugLevel();
   }
   return res;
}


/*-----------------------------------------------------------------*/
/*!
    write the object to the stream
   @param stream (inout) output stream
*/
/*-----------------------------------------------------------------*/
bool SCSCP_procedurecall::Write(OML_outputstream& stream) const
{
 SCSCP_getdebugger()<<"\nSCSCP_procedurecall::Write() - entree\n";
 
 stream.WriteSCSCPStartMsg();
 
 OML_OMOBJ::WriteBeginTag(stream);
 stream.WriteBeginTagOMATTR();
 
 //write the options
 m_ProcedureOptions.Write(stream);
 
 stream.WriteBeginTagOMA();
 
 //write procedure name
 OML_OMS::Write(stream,SCSCP_tags::cdname, SCSCP_tags::procedure_call);
 stream.WriteBeginTagOMA();
 OML_OMS::Write(stream,GetCDName(), GetProcName());
 //write aguments
 m_ProcedureArguments.Write(stream);

 stream.WriteEndTagOMA();
 stream.WriteEndTagOMA();
 
 stream.WriteEndTagOMATTR();
 OML_OMOBJ::WriteEndTag(stream);
 bool res = stream.WriteSCSCPEndMsg();
 
 SCSCP_getdebugger()<<"\nSCSCP_procedurecall::Write() - sortie et retourne "<<res<<"\n";
 return res;
}

/*-----------------------------------------------------------------*/
/*! read the header of the object from the stream
   @param stream (inout) output stream
   @param iter (out) terator on the stream
   @param status (inout) status information 
*/ 
/*-----------------------------------------------------------------*/
bool SCSCP_procedurecall::ReadHeader(OML_inputstream& stream, OML_inputstream::iterator& iter,
                                     SCSCP_status* status)
{
 bool res = true;
 SCSCP_getdebugger()<<"\nSCSCP_procedurecall::ReadHeader() - entree\n";
 iter = NULL;
 res = stream.ReadLine(status); //start
 if (res) res = stream.ReadXmlDoc();
 if (res)
 {
  SCSCP_getdebugger()<<"\nSCSCP_procedurecall::ReadHeader() - ReadXmlDoc returns true\n";
  iter = stream.getiterator();
  res = stream.ReadTagOMOBJ(iter);
  
  iter = stream.GoToChild(iter);
  
  // parse attributes
  m_ProcedureOptions.SetCallID(SCSCP_call_ID("N/A"));
  m_ProcedureOptions.SetReturnType(SCSCP_option_return_object);
  m_ProcedureName = "";
  res = m_ProcedureOptions.Read(stream, iter);
  //parse the name of the procedure
  res = stream.ReadTagOMA(iter, false);
  if (res)
  {
   OML_inputstream::iterator iterchild = stream.GoToChild(iter);
   stream.ReadPairOMSOMSTR(iterchild, SCSCP_tags::cdname, SCSCP_tags::procedure_call, m_ProcedureName);
   iter = iterchild;
  }
 }
 SCSCP_getdebugger()<<"\nSCSCP_procedurecall::ReadHeader("<<SCSCP_status_is(status)<<") - sortie et retourne "<<res<<"\n";
 return res;
}

/*-----------------------------------------------------------------*/
/*! read the object from the stream
   @param stream (inout) output stream
   @param status (inout) status information
*/
/*-----------------------------------------------------------------*/
bool SCSCP_procedurecall::Read(OML_inputstream& stream, SCSCP_status *status)
{
 bool res;
 OML_inputstream::iterator iter;
 SCSCP_getdebugger()<<"\nSCSCP_procedurecall::Read() - entree\n";

 res = ReadHeader(stream, iter, status);
 if (res && iter) res = m_ProcedureArguments.Read(stream, iter, m_ProcedureName);

 SCSCP_getdebugger()<<"\nSCSCP_procedurecall::Read() - sortie et retourne "<<res<<"\n";
 return res;
}

/*-----------------------------------------------------------------*/
/*! Display debug information about SCSCP_procedurecall
*/
/*-----------------------------------------------------------------*/
void SCSCP_procedurecall::Debug() const
{
 SCSCP_getdebugger()<<"SCSCP_procedurecall object\n";
 SCSCP_getdebugger()<<"name='"<<m_ProcedureName<<"'\n";
 SCSCP_getdebugger()<<"end of SCSCP_procedurecall object\n";
}

/*-----------------------------------------------------------------*/
/*! send a procedure call to the SCSCP server with some options 
    The parameter of the procedure call are written 
    by the callback function "callbackwriteargs"
 @param client (inout) scscp client
 @param options (in) options of the call
 @param callbackwriteargs (in) callback function 
             to write the parameters of the procedure call
 @param param (inout) opaque data given to callbackwriteargs
 @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_procedurecall_Write(SCSCP_Fileclient* client, SCSCP_procedurecalloptions* options, 
                          int (*callbackwriteargs)(SCSCP_Fileclient** client, void *param, SCSCP_status* status),
                          void *param, SCSCP_status* status)
{
 int res;
 SCSCP_getdebugger()<<"\nSCSCP_procedurecall::Write() - entree\n";
 
 OML_Fileoutputstream stream(client->GetFileio(), true);
 
 stream.WriteSCSCPStartMsg();
 
 OML_OMOBJ::WriteBeginTag(stream);
 stream.WriteBeginTagOMATTR();
 
 //write the options
 if (options) options->Write(stream);
 
 stream.WriteBeginTagOMA();
 
 //write procedure name
 OML_OMS::Write(stream,SCSCP_tags::cdname, SCSCP_tags::procedure_call);
 
 res = callbackwriteargs(&client, param, status);
 if (res)
 {
  stream.WriteEndTagOMA();
  stream.WriteEndTagOMATTR();
  OML_OMOBJ::WriteEndTag(stream);
  res = stream.WriteSCSCPEndMsg();
 }
 
 SCSCP_getdebugger()<<"\nSCSCP_procedurecall::Write() - sortie et retourne "<<res<<"\n";
 return res;
}

/*-----------------------------------------------------------------*/
/*!  read the attribute of the answer for the procedure call 
    and the type of the answer message 
 @param client (inout) scscp client
 @param options (inout) options of the procedure return
 @param msgtype (out) type of returned message
 @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_sc_callrecvheader(SCSCP_socketclient* client, SCSCP_returnoptions* options,
                            SCSCP_msgtype *msgtype, SCSCP_status* status)
{
 int res;
 SCSCP_debugprint("SCSCP_sc_callrecvheader() - start\n");
 SCSCP_status_seterror(status, SCSCP_STATUS_OK);
 res = SCSCP_sc_checkNULL(client, status);
 if (res)
 {
  SCSCP_socketclient pclient = *client;
  SCSCP_optionsprocedurereturn localoptions;
  res = pclient->Read(localoptions, pclient->GetIter(), *msgtype, status);
  if (res && options!=SCSCP_RETURNOPTIONS_IGNORE) **options = localoptions;
 }

 SCSCP_debugprint("SCSCP_sc_callrecvheader(%d) - returns %d\n",SCSCP_status_is(status), res);
 return res;
}                            

/*-----------------------------------------------------------------*/
/*! read the answer of the procedure call and store it in a string buffer 
 @param client (inout) scscp client
 @param options (inout) options of the procedure return
 @param msgtype (out) type of returned message
 @param openmathbuffer (out) openmath buffer. 
                 This buffer must be freed with 'free'
 @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_sc_callrecvstr(SCSCP_socketclient* client, SCSCP_returnoptions* options,
                         SCSCP_msgtype *msgtype, char **openmathbuffer, SCSCP_status* status)
{
 int res = SCSCP_sc_callrecvheader(client, options, msgtype, status);
 if (res)
 {
  SCSCP_socketclient pclient = *client;
  *openmathbuffer = pclient->GetInputStream()->GetRawString(pclient->GetIter());
 }
 return res;
}

/*-----------------------------------------------------------------*/
/*! read the content of a "procedure terminated message" 
    and the error message is stored in  errortype and messagebuffer 
 @param client (inout) scscp client
 @param errortype (out) type of error
 @param messagebuffer (out) openmath buffer. 
                 This buffer must be freed with 'free'
 @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_sc_callrecvterminated(SCSCP_socketclient* client, SCSCP_error* errortype, 
                                char **messagebuffer, SCSCP_status* status)
{
 int res = SCSCP_sc_checkNULL(client, status);
 if (res)
 {
  SCSCP_socketclient pclient = *client;
  SCSCP_procedureterminatedoptions options;
  SCSCP_procedureterminated localterm(options);
  res = pclient->Read(localterm, pclient->GetIter());
  if (res)
  {
   *messagebuffer = SCSCP_strdup(localterm.GetErrorMsg(), status);
   *errortype = localterm.GetError();
  }
 }
 return res;
}

/*-----------------------------------------------------------------*/
/*! read the content of a "procedure completed message" 
    and store it in openmathbuffer 
 @param client (inout) scscp client
 @param openmathbuffer (out) openmath buffer. 
                 This buffer must be freed with 'free'
 @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_sc_callrecvcompleted(SCSCP_socketclient* client, char **openmathbuffer,
                               SCSCP_status* status)
{
 int res = SCSCP_sc_checkNULL(client, status);
 if (res)
 {
  SCSCP_socketclient pclient = *client;
  *openmathbuffer = pclient->GetInputStream()->GetRawString(pclient->GetIter());
 }
 return res;
}



