/*-----------------------------------------------------------------*/
/*! 
  \file scscpprocedurecompleted.h
  \brief SCSCP message : procedure completed.
   class SCSCP_procedurecompletedoptions, SCSCP_procedurecompletedoptions
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __SCSCPPROCEDURECOMPLETED_H__
#define __SCSCPPROCEDURECOMPLETED_H__

#include "scscptypes.h"
#include "scscpoptions.h"
#include "scscpprocedurecall.h"


/*-----------------------------------------------------------------*/
/*!
    @class SCSCP_procedurecompletedoptions
    \brief  classe pour gerer les options du  message Procedure completed
*/
/*-----------------------------------------------------------------*/
typedef  SCSCP_optionsprocedurereturn SCSCP_procedurecompletedoptions;


/*-----------------------------------------------------------------*/
/*!
    @class SCSCP_procedurecompleted
    \brief  classe pour gerer un message Procedure completed
*/
/*-----------------------------------------------------------------*/
class SCSCP_procedurecompleted
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_procedurecompleted(SCSCP_procedurecompletedoptions& options, SCSCP_procedurearguments& args);
   virtual ~SCSCP_procedurecompleted() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! set the return cookie value
   void SetReturnCookie(const_SCSCP_string cookie) { m_cookie = cookie; };
   //! return the cookie value
   const_SCSCP_string GetReturnCookie() const { return m_cookie; };
   //! return the options
   SCSCP_optionsprocedurereturn& GetOptions() { return m_ProcedureOptions; };
   //! set the options
   void SetOptions(SCSCP_optionsprocedurereturn& opt) {m_ProcedureOptions = opt; };
   //! read the end of the message
   bool Read(OML_inputstream& stream,  OML_inputstream::iterator& iterchild, bool mustbecookie);
   
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////
   //! write the object to the stream
   virtual bool Write(OML_outputstream& stream) const ;
   //! write the return object to the stream if m_cookie =NULL
   virtual bool WriteObject(OML_outputstream& stream) const { return true; };

private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_procedurecompletedoptions& m_ProcedureOptions; //!< options of the procedure
   SCSCP_procedurearguments&        m_Resultats; //!< results of the procedure
   const_SCSCP_string               m_cookie; //!< return cookie value

   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   
};

class SCSCP_Fileclient;
int SCSCP_procedurecompleted_Write(SCSCP_Fileclient* client, SCSCP_procedurecompletedoptions* options, 
                          int (*callbackwriteargs)(SCSCP_Fileclient** client, void *param, SCSCP_status* status),
                          void *param, SCSCP_status* status);

#endif /*__SCSCPPROCEDURECOMPLETED_H__*/
