/*-----------------------------------------------------------------*/
/*! 
  \file scscpprocedureterminated.cxx
  \brief SCSCP message : procedure terminated.
   class SCSCP_procedureterminatedoptions, SCSCP_procedureterminated
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#if HAVE_CONFIG_H
#include "scscpconfig.h"
#endif /*HAVE_CONFIG_H*/
#include <stdio.h>
#if HAVE_STRING_H
#include <string.h>
#endif
#include "scscptypes.h"
#include "scscpinternal.h"
#include "scscpprocedureterminated.h"
#include "scscpdebug.h"
#include "omloms.h"
#include "omlome.h"
#include "omlomobj.h"
#include "scscptags.h"

/*-----------------------------------------------------------------*/
/*! constructor
   @param options (inout) options of the procedure completed
   @param error (in) error type
   @param errormsg (in) error message
   @param symbolname (in) name of the symbol
   @param cdname (in) name of the CD
*/
/*-----------------------------------------------------------------*/
SCSCP_procedureterminated::SCSCP_procedureterminated(SCSCP_procedureterminatedoptions& options,
                             SCSCP_error error, const_SCSCP_string errormsg,
                             const_SCSCP_string symbolname, const_SCSCP_string cdname)
    : m_ProcedureOptions(options), m_errortype(error), m_errormsg(errormsg), 
      m_symbolname(symbolname), m_cdname(cdname)
{
}

/*-----------------------------------------------------------------*/
/*! constructor
   @param options (inout) options of the procedure completed
   @param errormsg (in) error message
*/
/*-----------------------------------------------------------------*/
SCSCP_procedureterminated::SCSCP_procedureterminated(SCSCP_procedureterminatedoptions& options,
                              const_SCSCP_string errormsg)
    : m_ProcedureOptions(options),  m_errortype(SCSCP_error_system_specific), 
      m_errormsg(errormsg), m_symbolname(NULL), m_cdname(NULL)
{
}

/*-----------------------------------------------------------------*/
/*! write the object to the stream
   @param stream (inout) output stream
*/
/*-----------------------------------------------------------------*/
bool SCSCP_procedureterminated::Write(OML_outputstream& stream) const
{
 SCSCP_getdebugger()<<"\nSCSCP_procedureterminated::Write() - entree\n";
 
 stream.WriteSCSCPStartMsg();
 
 OML_OMOBJ::WriteBeginTag(stream);
 stream.WriteBeginTagOMATTR();
 
 //write the options
 m_ProcedureOptions.Write(stream);
 
 stream.WriteBeginTagOMA();
 
 //write procedure name
 OML_OMS::Write(stream, SCSCP_tags::cdname, SCSCP_tags::procedure_terminated);
 
 const_SCSCP_string serrortype = GetErrorTypeString();
 OML_OME::Write(stream, m_cdname!=NULL?m_cdname:SCSCP_tags::cdname, serrortype, m_errormsg);

 stream.WriteEndTagOMA();
 
 stream.WriteEndTagOMATTR();
 OML_OMOBJ::WriteEndTag(stream);
 bool res = stream.WriteSCSCPEndMsg();
 
 SCSCP_getdebugger()<<"\nSCSCP_procedureterminated::Write() - sortie et retourne "<<res<<"\n";
 return res;
}

/*-----------------------------------------------------------------*/
/*! read the object from the stream
   @param stream (inout) input stream
   @param iter (inout) iterator
*/
/*-----------------------------------------------------------------*/
bool SCSCP_procedureterminated::Read(OML_inputstream& stream,  OML_inputstream::iterator& iter)
{
 bool res;
 SCSCP_getdebugger()<<"\nSCSCP_procedureterminated::Read() - entree2\n";
 OML_inputstream::const_OMLchar errorname;

 res = OML_OME::Read(stream, iter,errorname, m_errormsg);
 
 if (strcmp(errorname, SCSCP_tags::error_memory) ==0) m_errortype = SCSCP_error_memory;
 else  if (strcmp(errorname, SCSCP_tags::error_runtime) ==0) m_errortype = SCSCP_error_runtime;
 else  if (strcmp(errorname, SCSCP_tags::error_system_specific) ==0) m_errortype = SCSCP_error_system_specific;
 
 SCSCP_getdebugger()<<"\nSCSCP_procedureterminated::Read() - sortie et retourne "<<res<<"\n";
 return res;
}

/*-----------------------------------------------------------------*/
/*! return the name of error type
*/
/*-----------------------------------------------------------------*/
const_SCSCP_string SCSCP_procedureterminated::GetErrorTypeString() const
{
 const_SCSCP_string serrortype;
 switch(m_errortype)
 {
  case SCSCP_error_memory :  serrortype = SCSCP_tags::error_memory ; break;
  case SCSCP_error_runtime : serrortype = SCSCP_tags::error_runtime ;  break;
  case SCSCP_error_system_specific : serrortype = SCSCP_tags::error_system_specific; break;
  case SCSCP_error_other : serrortype = m_symbolname; break;
  default : serrortype = m_symbolname; break;
 }
 return serrortype;
}
 

