/*-----------------------------------------------------------------*/
/*! 
  \file scscpprocedureterminated.h
  \brief SCSCP message : procedure terminated.
   class SCSCP_procedureterminatedoptions, SCSCP_procedureterminated
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __SCSCPPROCEDURETERMINATED_H__
#define __SCSCPPROCEDURETERMINATED_H__

#include "scscptypes.h"
#include "scscpcallid.h"
#include "scscpoptions.h"

/*-----------------------------------------------------------------*/
/*!
    @class SCSCP_procedureterminatedoptions
    \brief  classe pour gerer les options du  message Procedure terminated
*/
/*-----------------------------------------------------------------*/
typedef  SCSCP_optionsprocedurereturn SCSCP_procedureterminatedoptions;

/*-----------------------------------------------------------------*/
/*!
    @class SCSCP_procedureterminated
    \brief  classe pour gerer un message Procedure termiated
*/
/*-----------------------------------------------------------------*/
class SCSCP_procedureterminated
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_procedureterminated(SCSCP_procedureterminatedoptions& options, SCSCP_error error,
                             const_SCSCP_string errormsg=NULL,
                             const_SCSCP_string symbolname=NULL,
                             const_SCSCP_string cdname=NULL);
   SCSCP_procedureterminated(SCSCP_procedureterminatedoptions& options, const_SCSCP_string errormsg=NULL);
   virtual ~SCSCP_procedureterminated() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! return the options
   SCSCP_procedureterminatedoptions& GetOptions() { return m_ProcedureOptions; };
   //! set the options
   void SetOptions(SCSCP_procedureterminatedoptions& opt) {m_ProcedureOptions = opt; };
   //! read the end of the message
   bool Read(OML_inputstream& stream,  OML_inputstream::iterator& iterchild);
   //! return the name of error type
   const_SCSCP_string GetErrorTypeString() const; 
   //! return th error message
   const_SCSCP_string GetErrorMsg() const { return m_errormsg; };
   //! return the error
   const SCSCP_error& GetError() const { return m_errortype; };
    
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////
   //! write the object to the stream
   virtual bool Write(OML_outputstream& stream) const ;

private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_procedureterminatedoptions& m_ProcedureOptions; //!< options of the procedure
   SCSCP_error                      m_errortype; //!< error type
   const_SCSCP_string               m_errormsg;   //!< error message
   const_SCSCP_string               m_symbolname;   //!< symbol name
   const_SCSCP_string               m_cdname;   //!< cd name
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   
};


#endif /*__SCSCPPROCEDURETERMINATED_H__*/
