/*-----------------------------------------------------------------*/
/*! 
  \file scscpserver.h
  \brief SCSCP server.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __SCSCPSERVER_H__
#define __SCSCPSERVER_H__


/*-----------------------------------------------------------------*/
/*!
    @class SCSCP_server
    \brief  classe de base pour gerer un serveur SCSCP
*/
/*-----------------------------------------------------------------*/
class SCSCP_server
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_server() : m_allowedversions(NULL), m_allowedversionscount(0), m_servicename(NULL),
                    m_serviceversion(NULL), m_serviceid(NULL), m_errno(0) {};
   virtual ~SCSCP_server() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! return the error errno on failure
   int GetError() const { return m_errno; };

   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////
   //! return the output stream with the connection
   virtual OML_outputstream* GetOutputStream() { return NULL; };
    //! return the input stream with the connection
   virtual OML_inputstream*  GetInputStream() { return NULL; };

   ////////////////////////////////////////////////////////////////////////////
   // attributes
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_string *m_allowedversions; //!< allowed versions
   size_t m_allowedversionscount; //!< number of elements of m_allowedversions
   SCSCP_string m_servicename; //!< name of the service
   SCSCP_string m_serviceversion; //!< version of the service
   SCSCP_string m_serviceid; //!< unique identifier of the service

protected:
   ////////////////////////////////////////////////////////////////////////////
   // operation protected
   ////////////////////////////////////////////////////////////////////////////
   //! set the error errno on failure
   void seterror(int err) { m_errno = err; }; 

private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   int m_errno; //! errno value

};

int SCSCP_ss_clearallowedversions(SCSCP_server* server, SCSCP_status* status);
int SCSCP_ss_setservice(SCSCP_server* server, const char *servicename, const char *serviceversion, const char *serviceid, SCSCP_status* status);

#endif /*__SCSCPSERVER_H__*/
