/*-----------------------------------------------------------------*/
/*! 
  \file scscpservertest.c
  \brief default server tests
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
 This library is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
*/
/*-----------------------------------------------------------------*/

#if HAVE_CONFIG_H
#include "scscpconfig.h"
#endif /*HAVE_CONFIG_H*/
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include "scscp.h"


#if defined(SIGUSR2)
#define SIGNALINTERRUPT SIGUSR2
#else
#define SIGNALINTERRUPT SIGINT
#endif

/* SCSCP server  */
static SCSCP_socketserver server;

int  processcall(SCSCP_incomingclient *incomingclient, SCSCP_status *status, int typeanswer);


/*-----------------------------------------------------------------*/
/*! stop the server when it receive SIGUSR2  */
/*-----------------------------------------------------------------*/
void stopserver(int sig)
{

 SCSCP_status status;

 printf("receive SIGUSR2\n"); 
 /* close the connection */
 SCSCP_ss_close(&server, &status);

 /* destroy the server */
 SCSCP_ss_clear(&server, &status);
 exit(0); 
}

/*-----------------------------------------------------------------*/
/*! start the server and listen on default port */
/*-----------------------------------------------------------------*/
int runserver(int typeanswer)
{
 SCSCP_status status;
 int res;
 SCSCP_incomingclient incomingclient;
 int port = SCSCP_PROTOCOL_DEFAULTPORT;
 
 printf("server thread started\n");
 /* register stopserver for the signal SIGUSR2 */
 signal(SIGNALINTERRUPT, stopserver);
 
 /*initialization of the server */
 res = SCSCP_ss_init(&server, &status, "MYCAS","1","myid", SCSCP_PROTOCOL_VERSION_1_1, 
               SCSCP_PROTOCOL_VERSION_1_2, "1.001", NULL); 
               
 if (typeanswer==5) port =SCSCP_PROTOCOL_DEFAULTPORT+1;
 if (res) res = SCSCP_ss_listen(&server, port, 0, &status); 
 
 if (res)
 {
     while (res!=0 && SCSCP_ss_acceptclient(&server, &incomingclient, &status))
      {
       printf("new client accepted !\n");
       while (res>0) res = processcall(&incomingclient, &status,typeanswer);
       SCSCP_ss_closeincoming(&incomingclient, &status);
      }
 }
 printf("destroy server\n");
 /* destroy the server */
 SCSCP_ss_close(&server, &status);
 res = SCSCP_ss_clear(&server, &status);
 printf("server thread crash => exit with value 1\n");
 return 1; 
}

/*-----------------------------------------------------------------*/
/*! process the procedure call */
/*-----------------------------------------------------------------*/
int  processcall(SCSCP_incomingclient *incomingclient, SCSCP_status *status,int typeanswer)
{
 int res, resopt;
 SCSCP_calloptions options;
 SCSCP_returnoptions returnopt;
 SCSCP_msgtype msgtype;
 size_t timeusage, memsize;
 int debuglevel;
 SCSCP_option_return returntype;
 
 res = SCSCP_co_init(&options, status);
 res = SCSCP_ro_init(&returnopt, status);
 if (res) 
 {
     res = SCSCP_ss_callrecvheader(incomingclient, &options, &msgtype, status);
     if (res)
     {
      const char *callid;
      if (!SCSCP_co_get_callid(&options, &callid, status)) callid="N/A client id";
      switch(msgtype)
      {
       case SCSCP_msgtype_ProcedureCall : 
            printf("Procedure Call\n");
            switch(typeanswer)
            {
             case 0: 
               res = SCSCP_ss_sendcompletedstr(incomingclient, SCSCP_RETURNOPTIONS_DEFAULT, 
                              "<OMI>0</OMI>", status); 
               break;
               
             case 1: 
               SCSCP_ro_set_runtime(&returnopt, 55, status);
               SCSCP_ro_set_memory(&returnopt, 100, status);
               SCSCP_ro_set_callid(&returnopt, callid, status);
               res = SCSCP_ss_sendcompletedstr(incomingclient, &returnopt, 
                              "<OMI>0</OMI>", status); 
               if (res==0) printf("SCSCP_ss_sendcompletedstr failed! status=%d.\n",SCSCP_status_is(status));
               break;

             case 2: 
               res = SCSCP_ss_sendcompletedstr(incomingclient, SCSCP_RETURNOPTIONS_DEFAULT, 
                              "<OMS cd=\"alg1\" name=\"zero\" /><OMS cd=\"arith1\" name=\"abs\" />", status); 
               if (res==0) printf("SCSCP_ss_sendcompletedstr failed! status=%d.\n",SCSCP_status_is(status));
               break;               

             case 3: 
               res = SCSCP_ss_sendterminatedstr(incomingclient, SCSCP_RETURNOPTIONS_DEFAULT, SCSCP_error_system_specific,
                              "unknown symbol", status); 
               if (res==0) printf("SCSCP_ss_sendterminatedstr failed! status=%d.\n",SCSCP_status_is(status));
               break;
               
             case 4: 
               resopt = SCSCP_co_get_runtimelimit(&options, &timeusage, status);
               if (resopt) 
                  printf("runtime limit = %lld\n", (long long)timeusage);
               else 
               {
                if (SCSCP_status_is(status)==SCSCP_STATUS_USAGEUNKNOWNRUNTIME)
                  printf("runtime limit not available\n");
                else res = 0;
               }
               resopt = SCSCP_co_get_minmemory(&options, &memsize, status);
               if (resopt) 
                  printf("minimum memory = %lld\n", (long long)memsize);
               else 
               {
                if (SCSCP_status_is(status)==SCSCP_STATUS_USAGEUNKNOWNMINMEMORY)
                  printf("minimum memory not available\n");
                else res = 0;
               }
               
               resopt = SCSCP_co_get_maxmemory(&options, &memsize, status);
               if (resopt) 
                  printf("maximum memory = %lld\n", (long long)memsize);
               else 
               {
                if (SCSCP_status_is(status)==SCSCP_STATUS_USAGEUNKNOWNMAXMEMORY)
                  printf("maximum memory not available\n");
                else res = 0;
               }
               
               resopt = SCSCP_co_get_debuglevel(&options, &debuglevel, status);
               if (resopt) 
                  printf("debug level = %d\n", debuglevel);
               else 
               {
                if (SCSCP_status_is(status)==SCSCP_STATUS_USAGEUNKNOWNDEBUGLEVEL)
                  printf("debug level not available\n");
                else res = 0;
               }
               
               resopt = SCSCP_co_get_returntype(&options, &returntype, status);
               if (resopt) 
                  printf("return type = %d\n", (int)returntype);
               else 
               {
                if (SCSCP_status_is(status)==SCSCP_STATUS_USAGEUNKNOWNRETURNTYPE)
                  printf("return type not available\n");
                else res = 0;
               }
               
               res = SCSCP_ss_sendcompletedstr(incomingclient, SCSCP_RETURNOPTIONS_DEFAULT,  "<OMI>0</OMI>", status); 
               if (res==0) printf("SCSCP_ss_sendcompletedstr failed! status=%d.\n",SCSCP_status_is(status));
               break;               
            }
            break;
       default :
            printf("unknown message type !!!\n"); 
            res = 0;
            break;
      }
     }
     else
     {
      printf("SCSCP_ss_callrecvheader failed! status=%d.\n",SCSCP_status_is(status));
      /* if client close the connection => return -1 */
      if (SCSCP_status_is(status)==SCSCP_STATUS_RECVQUIT) res = -1;
     }
  SCSCP_co_clear(&options, status);
  SCSCP_ro_clear(&returnopt, status);
 }
 else
 {
  printf("SCSCP_ro_init failed status=%d.\n",SCSCP_status_is(status));
 }
 return res;
}

/*-----------------------------------------------------------------*/
/*! main program */
/*-----------------------------------------------------------------*/
int main(int argc, char *argv[])
{
 printf("scscpservertest started with %d\n", atoi(argv[1]));
 return runserver(atoi(argv[1]));
}
